@@ -78,6 +78,9 @@ class EvaluationResult(BaseModel):
 
     success: bool
     reason: Optional[str] = None
+    diffs: Optional[List[Dict[str, Any]]] = None
+    expected_mutations: Optional[List[Dict[str, Any]]] = None
+    actual_mutations: Optional[List[Dict[str, Any]]] = None
 
 
 class PlatoTask(BaseModel):
@@ -96,7 +99,10 @@ class PlatoTask(BaseModel):
     prompt: str
     env_id: str
     start_url: str
+    dataset_name: Optional[str] = None
     eval_config: Optional[BasePlatoEvalConfig] = None
+    average_time: Optional[float] = None
+    average_steps: Optional[int] = None
 
     @field_serializer('eval_config')
     def serialize_eval_config(self, eval_config: Optional[BasePlatoEvalConfig], _info):
