@@ -1,5 +1,5 @@
 from datetime import datetime
-from typing import List, Optional, Dict, Any
+from typing import List, Optional, Dict, Any, Literal
 from plato.config import get_config
 from plato.models import PlatoTask, PlatoEnvironment
 from plato.exceptions import PlatoClientError
@@ -55,7 +55,7 @@ class Plato:
                 total=600,  # 10 minutes
                 connect=60,
                 sock_read=600,  # 10 minutes
-                sock_connect=60
+                sock_connect=60,
             )
             self._http_session = aiohttp.ClientSession(timeout=timeout)
         return self._http_session
@@ -67,12 +67,31 @@ class Plato:
             self._http_session = None
 
     async def make_environment(
-        self, env_id: str, open_page_on_start: bool = False
+        self,
+        env_id: str,
+        open_page_on_start: bool = False,
+        viewport_width: int = 1920,
+        viewport_height: int = 1080,
+        interface_type: Optional[Literal["browser"]] = "browser",
+        record_network_requests: bool = False,
+        record_actions: bool = False,
+        env_config: Optional[Dict[str, Any]] = None,
+        keepalive: bool = False,
+        alias: Optional[str] = None,
     ) -> PlatoEnvironment:
         """Create a new Plato environment for the given task.
 
         Args:
-            task (PlatoTask): The task to create an environment for.
+            env_id (str): The ID of the environment to create.
+            open_page_on_start (bool): Whether to open the page on start.
+            viewport_width (int): The width of the viewport.
+            viewport_height (int): The height of the viewport.
+            interface_type (Optional[str]): The type of interface to create. Defaults to None.
+            record_network_requests (bool): Whether to record network requests.
+            record_actions (bool): Whether to record actions.
+            env_config (Optional[Dict[str, Any]]): Environment configuration.
+            keepalive (bool): If true, jobs will not be killed due to heartbeat failures.
+            alias (Optional[str]): Optional alias for the job group.
 
         Returns:
             PlatoEnvironment: The created environment instance.
@@ -84,19 +103,28 @@ class Plato:
         async with self.http_session.post(
             f"{self.base_url}/env/make2",
             json={
-                "interface_type": "browser",
-                "interface_width": 1920,
-                "interface_height": 1080,
+                "interface_type": interface_type or "noop",
+                "interface_width": viewport_width,
+                "interface_height": viewport_height,
                 "source": "SDK",
                 "open_page_on_start": open_page_on_start,
                 "env_id": env_id,
-                "env_config": {},
+                "env_config": env_config or {},
+                "record_network_requests": record_network_requests,
+                "record_actions": record_actions,
+                "keepalive": keepalive,
+                "alias": alias,
             },
             headers=headers,
         ) as response:
             response.raise_for_status()
             data = await response.json()
-            return PlatoEnvironment(client=self, id=data["job_id"])
+            return PlatoEnvironment(
+                client=self,
+                env_id=env_id,
+                id=data["job_id"],
+                alias=data.get("alias"),
+            )
 
     async def get_job_status(self, job_id: str) -> Dict[str, Any]:
         """Get the status of a job.
@@ -159,17 +187,11 @@ class Plato:
             response.raise_for_status()
             return await response.json()
 
-    async def reset_environment(
-        self,
-        job_id: str,
-        task: Optional[PlatoTask] = None,
-        agent_version: Optional[str] = None,
-    ) -> Dict[str, Any]:
-        """Reset an environment with an optional new task.
+    async def backup_environment(self, job_id: str) -> Dict[str, Any]:
+        """Create a backup of an environment.
 
         Args:
-            job_id (str): The ID of the job to reset.
-            task (Optional[PlatoTask]): Optional new task for the environment.
+            job_id (str): The ID of the job to backup.
 
         Returns:
             Dict[str, Any]: The response from the server.
@@ -178,7 +200,39 @@ class Plato:
             aiohttp.ClientError: If the API request fails.
         """
         headers = {"X-API-Key": self.api_key}
-        body = {"test_case_public_id": task.public_id if task else None, "agent_version": agent_version}
+        async with self.http_session.post(
+            f"{self.base_url}/env/{job_id}/backup", headers=headers
+        ) as response:
+            response.raise_for_status()
+            return await response.json()
+
+    async def reset_environment(
+        self,
+        job_id: str,
+        task: Optional[PlatoTask] = None,
+        agent_version: Optional[str] = None,
+        load_authenticated: bool = False,
+    ) -> Dict[str, Any]:
+        """Reset an environment with an optional new task.
+
+        Args:
+            job_id (str): The ID of the job to reset.
+            task (Optional[PlatoTask]): Optional new task for the environment.
+            agent_version (Optional[str]): Optional agent version.
+            load_authenticated (bool): Whether to load authenticated browser state.
+
+        Returns:
+            Dict[str, Any]: The response from the server.
+
+        Raises:
+            aiohttp.ClientError: If the API request fails.
+        """
+        headers = {"X-API-Key": self.api_key}
+        body = {
+            "test_case_public_id": task.public_id if task else None,
+            "agent_version": agent_version,
+            "load_browser_state": load_authenticated,
+        }
         start_time = time.time()
         async with self.http_session.post(
             f"{self.base_url}/env/{job_id}/reset", headers=headers, json=body
@@ -296,7 +350,9 @@ class Plato:
             response.raise_for_status()
             return await response.json()
 
-    async def evaluate(self, session_id: str, agent_version: Optional[str] = None) -> Dict[str, Any]:
+    async def evaluate(
+        self, session_id: str, agent_version: Optional[str] = None
+    ) -> Dict[str, Any]:
         """Evaluate the environment.
 
         Args:
@@ -316,7 +372,7 @@ class Plato:
         session_id: str,
         evaluation_result: EvaluationResult,
         agent_version: Optional[str] = None,
-        mutations: Optional[List[dict]] = None
+        mutations: Optional[List[dict]] = None,
     ) -> Dict[str, Any]:
         """Post an evaluation result to the server.
 
@@ -328,7 +384,7 @@ class Plato:
             "success": evaluation_result.success,
             "reason": evaluation_result.reason,
             "agent_version": agent_version,
-            "mutations": mutations
+            "mutations": mutations,
         }
         headers = {"X-API-Key": self.api_key}
         async with self.http_session.post(
@@ -339,7 +395,9 @@ class Plato:
             response.raise_for_status()
             return await response.json()
 
-    async def log(self, session_id: str, log: dict, type: str = "info") -> Dict[str, Any]:
+    async def log(
+        self, session_id: str, log: dict, type: str = "info"
+    ) -> Dict[str, Any]:
         """Log a message to the server.
 
         Args:
@@ -349,12 +407,14 @@ class Plato:
         """
         headers = {"X-API-Key": self.api_key}
         async with self.http_session.post(
-            f"{self.base_url}/env/{session_id}/log", headers=headers, json={
+            f"{self.base_url}/env/{session_id}/log",
+            headers=headers,
+            json={
                 "source": "agent",
                 "type": type,
                 "message": log,
-                "timestamp": datetime.now().isoformat()
-            }
+                "timestamp": datetime.now().isoformat(),
+            },
         ) as response:
             response.raise_for_status()
             return await response.json()
@@ -381,21 +441,28 @@ class Plato:
         """
         headers = {"X-API-Key": self.api_key}
         async with self.http_session.get(
-            f"{self.base_url}/testcases?simulator_name={simulator_name}&page_size=1000&scoring_config_type=state_mutation_match", headers=headers
+            f"{self.base_url}/testcases?simulator_name={simulator_name}&page_size=1000",
+            headers=headers,
         ) as response:
             response.raise_for_status()
             res = await response.json()
             test_cases = res["testcases"]
-            return [PlatoTask(
-                public_id=t["publicId"],
-                name=t["name"],
-                prompt=t["prompt"],
-                start_url=t["startUrl"],
-                env_id=t["simulator"]["name"]
-            ) for t in test_cases]
+            return [
+                PlatoTask(
+                    public_id=t["publicId"],
+                    name=t["name"],
+                    prompt=t["prompt"],
+                    start_url=t["startUrl"],
+                    env_id=t["simulator"]["name"],
+                    average_time=t.get("averageTimeTaken"),
+                    average_steps=t.get("averageStepsTaken"),
+                )
+                for t in test_cases
+            ]
 
-
-    async def list_simulator_tasks_by_id(self, simulator_id: str) -> List[Dict[str, Any]]:
+    async def list_simulator_tasks_by_id(
+        self, simulator_id: str
+    ) -> List[Dict[str, Any]]:
         """Get all tasks associated with an environment.
 
         Args:
@@ -409,10 +476,32 @@ class Plato:
         """
         headers = {"X-API-Key": self.api_key}
         async with self.http_session.get(
-            f"{self.base_url}/testcases?simulator_id={simulator_id}&page_size=1000", headers=headers
+            f"{self.base_url}/testcases?simulator_id={simulator_id}&page_size=1000",
+            headers=headers,
         ) as response:
             response.raise_for_status()
             res = await response.json()
             return res["testcases"]
 
+    async def get_active_session(self, job_id: str) -> Dict[str, Any]:
+        """Get the active session for a job group.
 
+        Args:
+            job_id (str): The ID of the job group to get the active session for.
+
+        Returns:
+            Dict[str, Any]: The active session information.
+
+        Raises:
+            aiohttp.ClientError: If the API request fails.
+            PlatoClientError: If no active session is found.
+        """
+        headers = {"X-API-Key": self.api_key}
+        async with self.http_session.get(
+            f"{self.base_url}/env/{job_id}/active_session", headers=headers
+        ) as response:
+            response.raise_for_status()
+            data = await response.json()
+            if "error" in data:
+                raise PlatoClientError(data["error"])
+            return data
